/*
   Copyright (c) 2014 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "includes.h"

#include "svpwm.h"
#include "svpwm3level.h"
#include "pidq.h"
#include "vector.h"
#include "filter.h"
#include "angle.h"
#include "pll.h"
#include "fp_math.h"

extern void DSP28x_usDelay(unsigned long aCount);
extern void ResetSystemFsm(void);
extern void ResetPowerstageFsm(void);

#define INV_SQRT3_Q15 (int16_t)(1.0 / 1.73 * 32768.0)

PLX_SVPWM3L_Obj_t Svpwm3Obj;
PLX_SVPWM3L_Handle_t Svpwm3Handle;

PLX_PIDQ_Obj_t PiDqObj;
PLX_PIDQ_Handle_t PiDqHandle;

PLX_PLL_Obj_t PllObj;
PLX_PLL_Handle_t PllHandle;

struct CONTROL_VARS Ctrl;

/**
 * Initializes control task.
 */
void InitControl(void){
	uint16_t pwmPeriod = (uint16_t)(SYSCLK_HZ / 2 / PWM_HZ / 2);

	InitPWM(pwmPeriod);
	InitADC();

	Ctrl.tsk1.Fs = CONTROL_HZ;
	InitPuVars(Ctrl.tsk1.Fs);

	Svpwm3Handle = PLX_SVPWM3L_init(&Svpwm3Obj,sizeof(Svpwm3Obj));
	PLX_SVPWM3L_configure(Svpwm3Handle, pwmPeriod);

	PiDqHandle = PLX_PIDQ_init(&PiDqObj,sizeof(PiDqObj));

	PllHandle = PLX_PLL_init(&PllObj, sizeof(PllObj));

	DisableActuation();
	ResetControl();
}

void DisableActuation(void){
	// disable actuators
	EALLOW;
 	EPwm1Regs.TZFRC.bit.OST=1;
	EPwm2Regs.TZFRC.bit.OST=1;
	EPwm3Regs.TZFRC.bit.OST=1;
 	EDIS;

	Ctrl.tsk1.psEnabled = FALSE;
}

void EnableActuation(void){
	Ctrl.tsk1.psEnabled = TRUE;

#ifndef KEEP_POWERSTAGE_OFF
	// enable actuators
	EALLOW;
	EPwm1Regs.TZCLR.bit.OST=1;
	EPwm2Regs.TZCLR.bit.OST=1;
	EPwm3Regs.TZCLR.bit.OST=1;
	EDIS;
#endif
}

void ResetTask1(void)
{
	Ctrl.tsk1.stepCtr = 0;
	PLX_PIDQ_configure(PiDqHandle, PuVars.wrefTsScaling, Calib.KpD, Calib.KiD, Calib.KpQ, Calib.KiQ);

	PLX_PLL_configure(PllHandle, PuVars.frefTsScaling, Calib.KpPll, Calib.KiPll,
			-Calib.omegaMaxPll, Calib.omegaMaxPll,
			Calib.lockAnglePll, Calib.lockTicksPll, Calib.unlockAnglePll, Calib.unlockTicksPll);
	PLX_PLL_reset(PllHandle, Calib.omega0Pll);

	Ctrl.tsk1.gridPllIsLocked = FALSE;
	DisableActuation();
	Ctrl.tsk1.psEnableReq = FALSE;
	Ctrl.tsk1.psDisableReq = FALSE;

	// start out current regulator in specific state (until first update by task-2)
	SET_OPROBE(Ctrl.tsk1.IdSet, 0);
	SET_OPROBE(Ctrl.tsk1.IqSet, 0);
	SET_OPROBE(Ctrl.tsk1.VdFF, 0);
	SET_OPROBE(Ctrl.tsk1.VqFF, 0);
}

void ResetTask2(void)
{
	Ctrl.tsk2.active = 0;
	Ctrl.tsk2.stepCtr = 0;
	ResetSystemFsm();
	ResetPowerstageFsm();

	// for display purposes
	Ctrl.tsk2.thetaSms = 0;
	SET_OPROBE(Ctrl.tsk2.IdSet, 0);
	SET_OPROBE(Ctrl.tsk2.IqSet, 0);
	SET_OPROBE(Ctrl.tsk2.VdFF, 0);
	SET_OPROBE(Ctrl.tsk2.VqFF, 0);
}

/**
 * Called when control starts, initialize filters, regulators here.
 */
void ResetControl(void)
{
	Ctrl.backgroundTaskNumber = 0;
	ResetTask1();
	ResetTask2();
}

/**
 * Fast control task
 */
void ControlTask1(){
	Ctrl.tsk1.stepCtr++;

	// handle disabling/enabling of powerstage
	if(Ctrl.tsk1.psDisableReq){
		DisableActuation();
		Ctrl.tsk1.psDisableReq = FALSE;
		Ctrl.tsk1.psEnableReq = FALSE;
	} else if(Ctrl.tsk1.psEnableReq) {
		// flag will be reset further below
		PLX_PIDQ_reset(PiDqHandle);
	}

	// analog inputs
	SET_OPROBE(AdcOvrProbes.ADCRESULT0, AdcResult.ADCRESULT0);
	SET_OPROBE(AdcOvrProbes.ADCRESULT1, AdcResult.ADCRESULT1);
	SET_OPROBE(AdcOvrProbes.ADCRESULT2, AdcResult.ADCRESULT2);
	SET_OPROBE(AdcOvrProbes.ADCRESULT3, AdcResult.ADCRESULT3);
	SET_OPROBE(AdcOvrProbes.ADCRESULT4, AdcResult.ADCRESULT4);
	SET_OPROBE(AdcOvrProbes.ADCRESULT5, AdcResult.ADCRESULT5);
	SET_OPROBE(AdcOvrProbes.ADCRESULT6, AdcResult.ADCRESULT6);
	SET_OPROBE(AdcOvrProbes.ADCRESULT7, AdcResult.ADCRESULT7);

	// process current measurements
	int16_t iu, iv;
	iu = AdcOvrProbes.ADCRESULT0 - AIn.IuOffset;
	iu = Q_MUL(iu, Calib.currentGain, Q_IV);
	iv = AdcOvrProbes.ADCRESULT1 - AIn.IvOffset;
	iv = Q_MUL(iv, Calib.currentGain, Q_IV);

	SET_OPROBE(AIn.Iu, iu);
	SET_OPROBE(AIn.Iv, iv);
	SET_OPROBE(AIn.Iw, -(iu + iv));

	// process DC voltage measurements
	SET_OPROBE(AIn.Vdc,  Q_MUL(AdcOvrProbes.ADCRESULT2, Calib.dcVoltageGain, Q_IV));
	SET_OPROBE(AIn.Vmid, Q_MUL(AdcOvrProbes.ADCRESULT3, Calib.dcVoltageGain, Q_IV));

	// process grid voltage measurement
	int16_t vguv, vgvw;
	vguv = AdcOvrProbes.ADCRESULT4 - AIn.VguvOffset;
	vguv = Q_MUL(vguv, Calib.acVoltageGain, Q_IV);
	vgvw = AdcOvrProbes.ADCRESULT5 - AIn.VgvwOffset;
	vgvw = Q_MUL(vgvw, Calib.acVoltageGain, Q_IV);
	SET_OPROBE(AIn.Vguv, vguv);
	SET_OPROBE(AIn.Vgvw, vgvw);
	SET_OPROBE(AIn.Vgwu, -(vguv + vgvw));

	int16_t vga, vgb;
	vga = AIn.Vguv;
	vgb = AIn.Vgvw - AIn.Vgwu;
	vgb = Q_MUL(vgb, INV_SQRT3_Q15, 15);

	SET_OPROBE(Ctrl.tsk1.VgridA, vga);
	SET_OPROBE(Ctrl.tsk1.VgridB, vgb);

	// calculate phase voltage amplitude (correct for line-line measurement)
	int16_t vga2 = Q_MUL(Ctrl.tsk1.VgridA, Ctrl.tsk1.VgridA, Q_IV);
	int16_t vgb2 = Q_MUL(Ctrl.tsk1.VgridB, Ctrl.tsk1.VgridB, Q_IV);
	int16_t vg = Q_MUL(PLX_FPMATH_sqrtQ10(vga2+vgb2), INV_SQRT3_Q15, 15);
	SET_OPROBE(Ctrl.tsk1.Vgrid, vg);

	// PLL
	Ctrl.tsk1.gridVoltageAngle = PLX_FPMATH_atan2(Ctrl.tsk1.VgridA, Ctrl.tsk1.VgridB)
									- (PLX_ANGLE_R_ANGLE/12); // compensate for line-line measurement
	bool_t isLocked;
	PLX_PLL_udpate(PllHandle, Ctrl.tsk1.gridVoltageAngle, &Ctrl.tsk1.gridPllAngle, &Ctrl.tsk1.gridPllOmega, &isLocked);

	SET_OPROBE(Ctrl.tsk1.gridPllIsLocked, isLocked);

	// align synchronous frame
	SET_OPROBE(Ctrl.tsk1.fluxPosition, Ctrl.tsk1.gridPllAngle - (PLX_ANGLE_R_ANGLE/4)); // make q-axis produce real power

	// calculate maximal achievable phase voltage based on DC-link voltage (max is 1/sqrt(3))
	int16_t vsmax = Q_MUL(AIn.Vdc, INV_SQRT3_Q15, 15);
	// apply margin for current regulator
	vsmax = Q_MUL(vsmax, (int16_t)(0.95 * 32768.0), 15);
	SET_OPROBE(Ctrl.tsk1.Vsmax, vsmax);

	// current clark transformation
	int16_t ia, ib;
	ia = AIn.Iu; // (2*AIn.Iu - AIn.Iv - AIn.Iw)/3
	ib = AIn.Iv - AIn.Iw;
	ib = Q_MUL(ib, INV_SQRT3_Q15, 15);

	SET_OPROBE(Ctrl.tsk1.Ia, ia);
	SET_OPROBE(Ctrl.tsk1.Ib, ib);

	// transform currents into rotating frame
	int16_t id, iq;
	PLX_FPMATH_lookupSinAndCos(Ctrl.tsk1.fluxPosition, &Ctrl.tsk1.fluxPosSin, &Ctrl.tsk1.fluxPosCos);
	PLX_VECT_parkRot(Ctrl.tsk1.Ia, Ctrl.tsk1.Ib, &id, &iq, Ctrl.tsk1.fluxPosSin, Ctrl.tsk1.fluxPosCos);
	SET_OPROBE(Ctrl.tsk1.Id, id);
	SET_OPROBE(Ctrl.tsk1.Iq, iq);

	// determine errors
	SET_OPROBE(Ctrl.tsk1.IdErr, Ctrl.tsk1.IdSet - Ctrl.tsk1.Id);
	SET_OPROBE(Ctrl.tsk1.IqErr, Ctrl.tsk1.IqSet - Ctrl.tsk1.Iq);

	// update decoupling gains of current regulator (could happen at a lower rate, e.g. 1000 Hz)
	SET_OPROBE(Ctrl.tsk1.we, Ctrl.tsk1.gridPllOmega);
	PLX_PIDQ_updateCrossCouplingGains(PiDqHandle,  Ctrl.tsk1.we);
	PLX_PIDQ_applyCrossCouplingGains(PiDqHandle); // this function must not be interrupted!

	// calculate regulator output
	int16_t vd, vq;
	PLX_PIDQ_calculateOutput(PiDqHandle, Ctrl.tsk1.IdErr, Ctrl.tsk1.IqErr, &vd, &vq);

	// add feedforward term
	vd += Ctrl.tsk1.VdFF;
	vq += Ctrl.tsk1.VqFF;

	// saturate output based on maximal allowable phase voltage
	PLX_VECT_satVector(Ctrl.tsk1.Vsmax, &vd, &vq);

	SET_OPROBE(Ctrl.tsk1.Vd, vd);
	SET_OPROBE(Ctrl.tsk1.Vq, vq);

	// transform voltages back into stationary frame
	int16_t va, vb;
	PLX_VECT_invParkRot(Ctrl.tsk1.Vd, Ctrl.tsk1.Vq, &va, &vb, Ctrl.tsk1.fluxPosSin, Ctrl.tsk1.fluxPosCos);
	SET_OPROBE(Ctrl.tsk1.Va, va);
	SET_OPROBE(Ctrl.tsk1.Vb, vb);

	// mid-point balancing (might make more sense to move this to task2 and filter voltages)
	SET_OPROBE(Ctrl.tsk1.VmidErr,  (AIn.Vdc >> 1) - AIn.Vmid);
	int16_t midErr = Ctrl.tsk1.VmidErr;

	if(midErr > Calib.MidErrSat)
	{
		midErr = Calib.MidErrSat; // saturate to prevent overflow
	}
	else if (midErr < -Calib.MidErrSat)
	{
		midErr = -Calib.MidErrSat; // saturate to prevent overflow
	}
	int16_t redVect2Adj = Q_MUL((midErr << 5), Calib.KpMid, 0);
	if(redVect2Adj > (int16_t)(0.5 * (double)(32768)))
	{
		redVect2Adj = (int16_t)(0.5 * (double)(32768));
	}
	else if(redVect2Adj < (int16_t)(-0.5 * (double)(32768)))
	{
		redVect2Adj = (int16_t)(-0.5 * (double)(32768));
	}
	SET_OPROBE(Ctrl.tsk1.redVector2Adj , redVect2Adj);

	// 3-level SVPWM
	int16_t svpwmLimitingGain;
	SET_OPROBE(Ctrl.tsk1.redVector1Split, (uint16_t)(0.5 * (double)(32768)));
	SET_OPROBE(Ctrl.tsk1.redVector2Split, (uint16_t)(0.5 * (double)(32768)- redVect2Adj)); // center of sub-hex

	PLX_SVPWM3L_calcCompareValues(Svpwm3Handle,
			Ctrl.tsk1.Va, Ctrl.tsk1.Vb, AIn.Vdc,
			Ctrl.tsk1.redVector1Split, Ctrl.tsk1.redVector2Split,
			&Ctrl.tsk1.Cmp0, &Ctrl.tsk1.Cmp1, &Ctrl.tsk1.Cmp2,
			&Ctrl.tsk1.Cmp3, &Ctrl.tsk1.Cmp4, &Ctrl.tsk1.Cmp5,
			&svpwmLimitingGain);

	// here is where we should update eEPWM

	// enable power-stage if it was off
	if(Ctrl.tsk1.psEnableReq)
	{
		Ctrl.tsk1.psEnableReq = FALSE;
		EnableActuation();
	}

	// for debugging
	PLX_SVPWM3L_getSector(Svpwm3Handle, &Ctrl.tsk1.subHexagon, &Ctrl.tsk1.zone);
	PLX_SVPWM3L_getVabT(Svpwm3Handle, &Ctrl.tsk1.VaT, &Ctrl.tsk1.VbT);
	PLX_SVPWM3L_getCmprT(Svpwm3Handle, &Ctrl.tsk1.Cmp0T, &Ctrl.tsk1.Cmp1T, &Ctrl.tsk1.Cmp2T);

	// apply SVPWM saturation to regulator outputs
	SET_OPROBE(Ctrl.tsk1.svpwmLimitingGain, svpwmLimitingGain);
	Ctrl.tsk1.VdSat = Q_MUL(Ctrl.tsk1.Vd, Ctrl.tsk1.svpwmLimitingGain, 14);
	Ctrl.tsk1.VqSat = Q_MUL(Ctrl.tsk1.Vq, Ctrl.tsk1.svpwmLimitingGain, 14);

	// update regulator integrator based on saturated regulator outputs
	PLX_PIDQ_updateIntegrator(PiDqHandle, Ctrl.tsk1.VdSat - Ctrl.tsk1.VdFF, Ctrl.tsk1.VqSat - Ctrl.tsk1.VqFF);
}

void ResetPowerstageFsm(void){
	Ctrl.tsk2.psFsm.state = PS_FSM_STATE_POWERUP;
	Ctrl.tsk2.psFsm.enable = FALSE;
}

void PowerstageFsm(void)
{
	switch(Ctrl.tsk2.psFsm.state)
	{
		// powerup
		case PS_FSM_STATE_POWERUP:
			if(!Ctrl.tsk2.psFsm.enable)
			{
				goto enter_PS_FSM_STATE_DISABLED;
			}
			break;

		// disabled
		enter_PS_FSM_STATE_DISABLED:
			// disable powerstage
			DINT;
			Ctrl.tsk1.psDisableReq = TRUE;
			Ctrl.tsk1.psEnableReq = FALSE;
			EINT;
			Ctrl.tsk2.psFsm.state = PS_FSM_STATE_DISABLED;
		break;
		case PS_FSM_STATE_DISABLED:
			if(Ctrl.tsk2.psFsm.enable)
			{
				goto enter_PS_FSM_STATE_ENABLING;
			}
			break;

		// enabling
		enter_PS_FSM_STATE_ENABLING:
			Ctrl.tsk2.psFsm.state = PS_FSM_STATE_ENABLING;
		break;
		case PS_FSM_STATE_ENABLING:
			// fallthrough

		// enabled
		enter_PS_FSM_STATE_ENABLED:
			// enable powerstage
			DINT;
			Ctrl.tsk1.psDisableReq = FALSE;
			Ctrl.tsk1.psEnableReq = TRUE;
			EINT;
			Ctrl.tsk2.psFsm.state = PS_FSM_STATE_ENABLED;
		break;
		case PS_FSM_STATE_ENABLED:
			// check for fault
			if(!Ctrl.tsk2.psFsm.enable)
			{
				goto enter_PS_FSM_STATE_DISABLED;
			}
			break;

		// fault
		enter_PS_FSM_STATE_FAULT:
			Ctrl.tsk2.psFsm.state = PS_FSM_STATE_FAULT;
		break;
		case PS_FSM_STATE_FAULT:
			break;

		// critical fault
		enter_PS_FSM_STATE_CRITICAL_FAULT:
			Ctrl.tsk2.psFsm.state = PS_FSM_STATE_CRITICAL_FAULT;
		break;
		case PS_FSM_STATE_CRITICAL_FAULT:
			break;

		// fault acknowledge
		enter_PS_FSM_STATE_FAULT_ACKN:
			Ctrl.tsk2.psFsm.state = PS_FSM_STATE_FAULT_ACKN;
			break;
		case PS_FSM_STATE_FAULT_ACKN:
			break;
	}
}

void ResetSystemFsm(void){
	Ctrl.tsk2.sysFsm.state = SYSTEM_FSM_STATE_POWERUP;
}

void SystemFsm(void)
{
	switch(Ctrl.tsk2.sysFsm.state)
	{
		case SYSTEM_FSM_STATE_POWERUP:
			// fallthrough

		enter_SYSTEM_FSM_STATE_CALIBRATE:
			Ctrl.tsk2.sysFsm.state = SYSTEM_FSM_STATE_CALIBRATE;
		break;
		case SYSTEM_FSM_STATE_CALIBRATE:
			goto enter_SYSTEM_FSM_STATE_PRECHARGE_DC;

		enter_SYSTEM_FSM_STATE_PRECHARGE_DC:
			Ctrl.tsk2.sysFsm.state = SYSTEM_FSM_STATE_PRECHARGE_DC;
			break;
		case SYSTEM_FSM_STATE_PRECHARGE_DC:
			if(AIn.Vdc >= Calib.VdcOn)
			{
				goto enter_SYSTEM_FSM_STATE_SYNCHRONIZE_AC;
			}
			break;

		enter_SYSTEM_FSM_STATE_SYNCHRONIZE_AC:
			Ctrl.tsk2.sysFsm.state = SYSTEM_FSM_STATE_SYNCHRONIZE_AC;
			break;
		case SYSTEM_FSM_STATE_SYNCHRONIZE_AC:
			if(AIn.Vdc <= Calib.VdcOff)
			{
				goto enter_SYSTEM_FSM_STATE_PRECHARGE_DC;
			}
			if((Ctrl.tsk1.Vgrid >= Calib.VGridMin) && Ctrl.tsk1.gridPllIsLocked)
			{
				goto enter_SYSTEM_FSM_STATE_READY;
			}
			break;

		enter_SYSTEM_FSM_STATE_READY:
			Ctrl.tsk2.sysFsm.state = SYSTEM_FSM_STATE_READY;
			SET_OPROBE(Ctrl.tsk2.VdFF, 0);
			SET_OPROBE(Ctrl.tsk2.VqFF, Ctrl.tsk1.Vgrid); // note: this assumes positive frequency!
			SET_OPROBE(Ctrl.tsk2.IdSet, 0);
			SET_OPROBE(Ctrl.tsk2.IqSet, 0);
			break;
		case SYSTEM_FSM_STATE_READY:
			if(AIn.Vdc <= Calib.VdcOff)
			{
				goto enter_SYSTEM_FSM_STATE_FAULT;
			}
			if((Ctrl.tsk1.Vgrid < Calib.VGridMin) || !Ctrl.tsk1.gridPllIsLocked)
			{
				goto enter_SYSTEM_FSM_STATE_FAULT;
			}
			goto enter_SYSTEM_FSM_STATE_ENABLED;

		enter_SYSTEM_FSM_STATE_ENABLED:
			Ctrl.tsk2.sysFsm.state = SYSTEM_FSM_STATE_ENABLED;

			break;
		case SYSTEM_FSM_STATE_ENABLED:
			if(AIn.Vdc <= Calib.VdcOff)
			{
				goto enter_SYSTEM_FSM_STATE_FAULT;
			}
			if((Ctrl.tsk1.Vgrid < Calib.VGridMin) || !Ctrl.tsk1.gridPllIsLocked)
			{
				goto enter_SYSTEM_FSM_STATE_FAULT;
			}

			SET_OPROBE(Ctrl.tsk2.IdDesired, 0);
			SET_OPROBE(Ctrl.tsk2.IqDesired, 0);
			{
				// Slip mode frequency shift
				int16_t dF = Calib.omega0Pll - Ctrl.tsk1.gridPllOmega;
				if(dF > Calib.SmsErrSat)
				{
					dF = Calib.SmsErrSat;
				}
				else if (dF < -Calib.SmsErrSat)
				{
					dF = -Calib.SmsErrSat;
				}
				SET_OPROBE(Ctrl.tsk2.thetaSms, Q_MUL(dF, Calib.KpSms, 10 - (16-10)));

				int16_t sin, cos;
				PLX_FPMATH_lookupSinAndCos(Ctrl.tsk2.thetaSms, &sin, &cos);
				int16_t idSet, iqSet;
				PLX_VECT_invParkRot(Ctrl.tsk2.IdDesired, Ctrl.tsk2.IqDesired, &idSet, &iqSet, sin, cos);

				SET_OPROBE(Ctrl.tsk2.IdSet, idSet);
				SET_OPROBE(Ctrl.tsk2.IqSet, iqSet);
			}
			break;

		enter_SYSTEM_FSM_STATE_FAULT:
			Ctrl.tsk2.sysFsm.state = SYSTEM_FSM_STATE_FAULT;
			break;
		case SYSTEM_FSM_STATE_FAULT:
			goto enter_SYSTEM_FSM_STATE_CRITICAL_FAULT; // no fault handling

		enter_SYSTEM_FSM_STATE_FAULT_ACKN:
			Ctrl.tsk2.sysFsm.state = SYSTEM_FSM_STATE_FAULT_ACKN;
			break;
		case SYSTEM_FSM_STATE_FAULT_ACKN:
			break;

		enter_SYSTEM_FSM_STATE_CRITICAL_FAULT:
			Ctrl.tsk2.sysFsm.state = SYSTEM_FSM_STATE_CRITICAL_FAULT;
			break;
		case SYSTEM_FSM_STATE_CRITICAL_FAULT:
			break;
	}
}


/**
 * Slow control task
 */
void ControlTask2()
{
	Ctrl.tsk2.active = TRUE;
	Ctrl.tsk2.stepCtr++;

	// system FSM checks makes sure DC and AC sides are ready
	SystemFsm();

	// determine if we should enable the powerstage
	Ctrl.tsk2.psFsm.enable = (Ctrl.tsk2.sysFsm.state == SYSTEM_FSM_STATE_ENABLED);

	// handle power-stage (actual enabling and disabling done in task1)
	PowerstageFsm();

	// export set-points
	DINT;
	SET_OPROBE(Ctrl.tsk1.IdSet, Ctrl.tsk2.IdSet);
	SET_OPROBE(Ctrl.tsk1.IqSet, Ctrl.tsk2.IqSet);
	SET_OPROBE(Ctrl.tsk1.VdFF, Ctrl.tsk2.VdFF);
	SET_OPROBE(Ctrl.tsk1.VqFF, Ctrl.tsk2.VqFF);
	EINT;

	PLX_DELAY_US(300); // we add some fake "load" to demonstrate Task2 being interrupted by Task1
	Ctrl.tsk2.active = FALSE;
}

/**
 * Control taks run in background loop
 */
void ControlBackground(void)
{
	switch(Ctrl.backgroundTaskNumber)
	{
		case 0:
			PLX_DELAY_US(300);
			break;

		case 1:
			PLX_DELAY_US(1000);
			break;

		case 2:
			PLX_DELAY_US(600);
			break;

	}
	Ctrl.backgroundTaskNumber++;
	if(Ctrl.backgroundTaskNumber > 2)
	{
		Ctrl.backgroundTaskNumber = 0;
	}
}



