/*
   Copyright (c) 2014 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include <xdc/std.h>
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Semaphore.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Swi.h>
#include <ti/sysbios/family/c28/Hwi.h>

#include "includes.h"

// linker addresses, needed to copy code from flash to ram
extern Uint16 RamfuncsLoadStart, RamfuncsLoadSize, RamfuncsRunStart;

// handles defined in rtos.cfg
extern const Semaphore_Handle BlinkSem;
extern const Swi_Handle Swi0;

// LED blink period/2
#define LED_BLINK_PERIOD_2 (500 * TICKS_PER_MS)

struct CTRL_DISPATCH_VARS CtrlDispatchVars;

Void ControlDispatcherHwi(UArg arg)
{
	PIL_beginInterruptCall();

#if defined(USE_CPU_TIMER_INT)
#error CPU Timer not supported.
#elif defined(USE_ADC_INT)
	AdcaRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;	// Clear ADCINT1 flag reinitialize for next SOC
#else
	EPwm1Regs.ETCLR.bit.INT = 1;  // Clear INT flag for this timer
#endif

	// fast control task
	ControlTask1();
	// slow control task
	if(CtrlDispatchVars.task2DispatchTimer == 0)
	{
		Swi_post(Swi0); // this will call ControlTask2();
	}
	CtrlDispatchVars.tickCounter++;

    // post the semaphore for LED blinking
	if(CtrlDispatchVars.ledBlinkTimer == 0){
		Semaphore_post(BlinkSem);
	}

	// update timers
	CtrlDispatchVars.task2DispatchTimer++;
	if(CtrlDispatchVars.task2DispatchTimer >= TASK2_PERIOD){
		CtrlDispatchVars.task2DispatchTimer = 0;
		CtrlDispatchVars.backgroundLoopCounter = 0;
	}

	CtrlDispatchVars.ledBlinkTimer++;
	if(CtrlDispatchVars.ledBlinkTimer >= LED_BLINK_PERIOD_2){
		CtrlDispatchVars.ledBlinkTimer = 0;
	}
}

/**
 * Software interrupt for slow control task
 */
Void ControlDispatcherSwi0(UArg arg0, UArg arg1)
{
	CtrlDispatchVars.task2Active = TRUE;
	ControlTask2();
	CtrlDispatchVars.task2Active = FALSE;
}

void ResetControlDispatcher()
{
	CtrlDispatchVars.tickCounter = 0;
	CtrlDispatchVars.task2DispatchTimer = 0;
	CtrlDispatchVars.ledBlinkTimer = 0;
	CtrlDispatchVars.task2Active = FALSE;
}

void InitControlDispatcher()
{
#if defined(USE_CPU_TIMER_INT)
#error CPU Timer not supported.
#elif defined(USE_ADC_INT)
	EALLOW;

	EPwm1Regs.ETSEL.bit.SOCAEN	= 1;		// Enable SOC on A group
	EPwm1Regs.ETSEL.bit.SOCASEL	= 2;		// Select SOC trigger on ePWM period
	EPwm1Regs.ETPS.bit.SOCAPRD 	= 1;		// Generate pulse on 1st event

	AdcaRegs.ADCINTSEL1N2.bit.INT1CONT  = 0;	// Disable ADCINT1 Continuous mode
	AdcaRegs.ADCINTSEL1N2.bit.INT1SEL 	= 0;    // setup EOC2 to trigger ADCINT1 (after Iu, Iv, Idc)
	AdcaRegs.ADCCTL1.bit.INTPULSEPOS	= 1;	// ADCINT1 trips after AdcResults latch
	AdcaRegs.ADCINTSEL1N2.bit.INT1E     = 1;	// Enable ADCINT1
	EDIS;
#else
	EPwm1Regs.ETSEL.bit.INTSEL = ET_CTR_PRD; // Select INT on period event
	EPwm1Regs.ETSEL.bit.INTEN = 1;           // Enable INT
	EPwm1Regs.ETPS.bit.INTPRD = ET_1ST; 	 // Generate interrupt on 1st event
#endif

	ResetControlDispatcher();
}

/**
 * LED blinking using "task" and "semaphore" concepts
 */
Void BlinkTaskFxn(Void){
	static char hi = 0;
    while (TRUE) {
    	// pend on semaphore
        Semaphore_pend(BlinkSem, BIOS_WAIT_FOREVER);
        // blink LED
        if(hi){
        	hi = 0;
        	GPIO_WritePin(CPU1_LED_GPIO, 1);
        } else {
        	hi = 1;
        	GPIO_WritePin(CPU1_LED_GPIO, 0);
        }
    }
}

Void BackgroundTaskFxn(Void)
{
	CtrlDispatchVars.backgroundLoopCounter++;
	PIL_backgroundCall();
	ControlBackground();
}

Void ResetHook(Void)
{
    EALLOW;
    ClkCfgRegs.CLKSRCCTL1.bit.XTALOFF=0;        // Turn on XTALOSC
    ClkCfgRegs.CLKSRCCTL1.bit.OSCCLKSRCSEL = 1; // Clk Src = XTAL
    EDIS;
}

Int main(void)
{
	memcpy(&RamfuncsRunStart, &RamfuncsLoadStart, (size_t)&RamfuncsLoadSize);
	// (important, or else F28x_usDelay will crash!)

	InitGpio();
	// configure IO for LED blinking
	GPIO_SetupPinMux(CPU1_LED_GPIO, GPIO_MUX_CPU1, 0);
	GPIO_SetupPinOptions(CPU1_LED_GPIO, GPIO_OUTPUT, GPIO_PUSHPULL);

	// configure serial line
	SCIInit(ONE_STOPBIT+ NO_PARITY + EIGHT_BITS, BAUD_RATE, LSPCLK_HZ);
	SCIWriteString("\n\rRS-232 Initialized.\n\r");

	// initialize PIL framework
	PIL_init();
	PIL_setLinkParams((unsigned char*)&PIL_D_Guid[0], (PIL_CommCallbackPtr_t)SCIPoll);
	PIL_setCtrlCallback((PIL_CtrlCallbackPtr_t)PilCallback);

	PilInitOverrideProbes();
	PilInitCalibrations();

	EALLOW;
	CpuSysRegs.PCLKCR0.bit.TBCLKSYNC = 0;      // Stop all the TB clocks
	EDIS;

	InitControl();
	InitControlDispatcher();

	EALLOW;
	CpuSysRegs.PCLKCR0.bit.TBCLKSYNC = 1;         // Start all the timers synced
	EDIS;

	// get ready for PIL
	PIL_requestReadyMode();

    BIOS_start();
    return(0);
}

